@@ -17,172 +17,6 @@ import itertools
 class ListOpError(Exception):
     pass
 
-class cat(TextOp):
-    r""" Return the content of the file with the path given in the input text
-
-    If a context dict is specified, the path is formatted with that context (str.format)
-
-    Args:
-        context (dict): The context to format the file path (Optionnal)
-
-    Yields:
-        str: the file content lines
-
-    Examples:
-        >>> open('/tmp/testfile.txt','w').write('here is the file content')
-        >>> '/tmp/testfile.txt' | cat()                 #doctest: +ELLIPSIS
-        <generator object extend_type_gen at ...>
-        >>> '/tmp/testfile.txt' | cat().tostr()
-        'here is the file content'
-        >>> '/tmp/testfile.txt' >> cat()
-        ['here is the file content']
-        >>> '/tmp/testfile.txt' | cat().upper().tostr()
-        'HERE IS THE FILE CONTENT'
-        >>> context = {'path':'/tmp/'}
-        >>> '{path}testfile.txt' | cat(context)                 #doctest: +ELLIPSIS
-        <generator object extend_type_gen at ...>
-        >>> '{path}testfile.txt' | cat(context).tostr()
-        'here is the file content'
-        >>> cat('/tmp/testfile.txt').s
-        'here is the file content'
-        >>> cat('/tmp/testfile.txt').upper().s
-        'HERE IS THE FILE CONTENT'
-        >>> cat('/tmp/testfile.txt').l
-        ['here is the file content']
-        >>> cat('/tmp/testfile.txt').g                 #doctest: +ELLIPSIS
-        <generator object extend_type_gen at ...>
-        >>> for line in cat('/tmp/testfile.txt'):
-        ...     print line
-        ...
-        here is the file content
-        >>> for bits in cat('/tmp/testfile.txt').grep('content').cut():
-        ...     print bits
-        ...
-        ['here', 'is', 'the', 'file', 'content']
-        >>> open('/tmp/testfile.txt','w').write('here is the file content\nanother line')
-        >>> '/tmp/testfile.txt' | cat().tostr()
-        'here is the file content\nanother line'
-        >>> '/tmp/testfile.txt' | cat().tolist()
-        ['here is the file content', 'another line']
-        >>> cat('/tmp/testfile.txt').s
-        'here is the file content\nanother line'
-        >>> cat('/tmp/testfile.txt').l
-        ['here is the file content', 'another line']
-        >>> context = {'path': '/tmp/'}
-        >>> cat('/{path}/testfile.txt',context).l
-        ['here is the file content', 'another line']
-        >>> for bits in cat('/tmp/testfile.txt').grep('content').cut():
-        ...     print bits
-        ...
-        ['here', 'is', 'the', 'file', 'content']
-    """
-    @classmethod
-    def op(cls,text, context = {},*args,**kwargs):
-        for path in cls._tolist(text):
-            if context:
-                path = path.format(**context)
-            path = os.path.expanduser(path)
-            if os.path.isfile(path) or os.path.islink(path):
-                with open(path) as fh:
-                    for line in fh:
-                        yield line.rstrip('\r\n')
-
-class run(TextOp):
-    r""" Run the command from the input text and return execution output
-
-    | This text operation use subprocess.Popen to call the command.
-    | If the command is a string, it will be executed within a shell.
-    | If the command is a list (the command and its arguments), the command is executed without a shell.
-    | If a context dict is specified, the command is formatted with that context (str.format)
-
-    Args:
-        context (dict): The context to format the command to run
-
-    Yields:
-        str: the execution output
-
-    Examples:
-        >>> cmd = 'mkdir -p /tmp/textops_tests_run;\
-        ... cd /tmp/textops_tests_run; touch f1 f2 f3; ls'
-        >>> print cmd | run().tostr()
-        f1
-        f2
-        f3
-        >>> print cmd >> run()
-        ['f1', 'f2', 'f3']
-        >>> print ['ls', '/tmp/textops_tests_run'] | run().tostr()
-        f1
-        f2
-        f3
-        >>> print ['ls', '{path}'] | run({'path':'/tmp/textops_tests_run'}).tostr()
-        f1
-        f2
-        f3
-    """
-    @classmethod
-    def op(cls,text, context = {},*args,**kwargs):
-        if isinstance(text, basestring):
-            if context:
-                text = text.format(**context)
-            p=subprocess.Popen(['sh','-c',text],stdout=subprocess.PIPE)
-        else:
-            if context:
-                text = [ t.format(**context) for t in text ]
-            p=subprocess.Popen(text,stdout=subprocess.PIPE)
-        while p.returncode is None:
-            (stdout, stderr) = p.communicate()
-            for line in stdout.splitlines():
-                yield line
-
-class mrun(TextOp):
-    r""" Run multiple commands from the input text and return execution output
-
-    | This works like :class:`textops.run` except that each line of the input text will be used as a command.
-    | The input text must be a list of strings (list, generator, or newline separated), \
-      not a list of lists. Commands will be executed inside a shell.
-    | If a context dict is specified, commands are formatted with that context (str.format)
-
-    Args:
-        context (dict): The context to format the command to run
-
-    Yields:
-        str: the execution output
-
-    Examples:
-        >>> cmds = 'mkdir -p /tmp/textops_tests_run\n'
-        >>> cmds+= 'cd /tmp/textops_tests_run;touch f1 f2 f3\n'
-        >>> cmds+= 'ls /tmp/textops_tests_run'
-        >>> print cmds | mrun().tostr()
-        f1
-        f2
-        f3
-        >>> cmds=['mkdir -p /tmp/textops_tests_run',
-        ... 'cd /tmp/textops_tests_run; touch f1 f2 f3']
-        >>> cmds.append('ls /tmp/textops_tests_run')
-        >>> print cmds | mrun().tostr()
-        f1
-        f2
-        f3
-        >>> print cmds >> mrun()
-        ['f1', 'f2', 'f3']
-        >>> cmds = ['ls {path}', 'echo "Cool !"']
-        >>> print cmds | mrun({'path':'/tmp/textops_tests_run'}).tostr()
-        f1
-        f2
-        f3
-        Cool !
-    """
-    @classmethod
-    def op(cls,text, context = {}, *args,**kwargs):
-        for cmd in cls._tolist(text):
-            if context:
-                cmd = cmd.format(**context)
-            p=subprocess.Popen(['sh','-c',cmd],stdout=subprocess.PIPE)
-            while p.returncode is None:
-                (stdout, stderr) = p.communicate()
-                for line in stdout.splitlines():
-                    yield line
-
 class grep(TextOp):
     r"""Select lines having a specified pattern
 
@@ -195,8 +29,11 @@ class grep(TextOp):
     will work for any kind of object.
 
     Args:
-        pattern (str): a regular expression string (case sensitive)
-        key (int or str): test only one column or one key (optional)
+        pattern (str): a regular expression string (case sensitive, Optionnal)
+        key (int or str): test the pattern only one column or one key (optional)
+        has_key (int or str): test only if the test_key is in the inner list or dict (optional)
+        attr (str): for list of objects, test the pattern on the object `attr` attribute (optional)
+        has_attr (int or str): For list of objects, test if the attribute `has_attr` exists (optional)
 
     Yields:
         str, list or dict: the filtered input text
@@ -235,10 +72,10 @@ class grep(TextOp):
     reverse = False
     pattern = ''
     @classmethod
-    def op(cls,text,pattern=None,key = None, has_key = None, *args,**kwargs):
+    def op(cls,text,pattern=None, key=None, has_key=None, attr=None, has_attr=None, *args,**kwargs):
         if pattern is None:
             pattern = cls.pattern
-        regex = re.compile(pattern,cls.flags)
+        regex = re.compile(pattern,cls.flags) if isinstance(pattern,basestring) else pattern
         for line in cls._tolist(text):
             try:
                 if isinstance(line,basestring):
@@ -247,11 +84,17 @@ class grep(TextOp):
                 elif has_key is not None:
                     if has_key in line != cls.reverse:  # kind of XOR with cls.reverse
                         yield line
-                elif key is None:
-                    if bool(regex.search(stru(line))) != cls.reverse:  # kind of XOR with cls.reverse
+                elif key is not None:
+                    if bool(regex.search(stru(line[key]))) != cls.reverse:  # kind of XOR with cls.reverse
+                        yield line
+                elif has_attr is not None:
+                    if hasattr(line,has_attr) != cls.reverse:  # kind of XOR with cls.reverse
+                        yield line
+                elif attr is not None:
+                    if bool(regex.search(stru(getattr(line,attr,'')))) != cls.reverse:  # kind of XOR with cls.reverse
                         yield line
                 else:
-                    if bool(regex.search(stru(line[key]))) != cls.reverse:  # kind of XOR with cls.reverse
+                    if bool(regex.search(stru(line))) != cls.reverse:  # kind of XOR with cls.reverse
                         yield line
             except (ValueError, TypeError, IndexError, KeyError):
                 pass
@@ -328,8 +171,11 @@ class grepc(TextOp):
     it counts lines matching the pattern.
 
     Args:
-        pattern (str): a regular expression string (case sensitive)
-        key (int or str): test only one column or one key (optional)
+        pattern (str): a regular expression string (case sensitive, Optionnal)
+        key (int or str): test the pattern only one column or one key (optional)
+        has_key (int or str): test only if the test_key is in the inner list or dict (optional)
+        attr (str): for list of objects, test the pattern on the object `attr` attribute (optional)
+        has_attr (int or str): For list of objects, test if the attribute `has_attr` exists (optional)
 
     Returns:
         int: the matched lines count
@@ -353,12 +199,12 @@ class grepc(TextOp):
     pattern = ''
     exit_on_found = False
     @classmethod
-    def op(cls,text,pattern=None,key = None, has_key = None,*args,**kwargs):
+    def op(cls,text,pattern=None,key = None, has_key = None, attr=None, has_attr=None,*args,**kwargs):
         if text is None:
             return 0
         if pattern is None:
             pattern = cls.pattern
-        regex = re.compile(pattern,cls.flags)
+        regex = re.compile(pattern,cls.flags) if isinstance(pattern,basestring) else pattern
         count = 0
         for line in cls._tolist(text):
             try:
@@ -372,13 +218,24 @@ class grepc(TextOp):
                         count += 1
                         if cls.exit_on_found:
                             break
-                elif key is None:
-                    if bool(regex.search(stru(line))) != cls.reverse:  # kind of XOR with cls.reverse
+                elif key is not None:
+                    if bool(regex.search(stru(line[key]))) != cls.reverse:  # kind of XOR with cls.reverse
+                        count += 1
+                        if cls.exit_on_found:
+                            break
+                elif has_attr is not None:
+                    if hasattr(line, has_attr) != cls.reverse:  # kind of XOR with cls.reverse
+                        count += 1
+                        if cls.exit_on_found:
+                            break
+                elif attr is not None:
+                    if bool(regex.search(
+                            stru(getattr(line, attr, '')))) != cls.reverse:  # kind of XOR with cls.reverse
                         count += 1
                         if cls.exit_on_found:
                             break
                 else:
-                    if bool(regex.search(stru(line[key]))) != cls.reverse:  # kind of XOR with cls.reverse
+                    if bool(regex.search(stru(line))) != cls.reverse:  # kind of XOR with cls.reverse
                         count += 1
                         if cls.exit_on_found:
                             break
@@ -788,6 +645,50 @@ class formatdicts(TextOp):
     def op(cls,items,format_str='{key} : {val}\n',join_str = '', context={}, defvalue='-',*args,**kwargs):
         return join_str.join([eformat(format_str,(),dict(context,**d),defvalue) for d in items ])
 
+class sortlists(TextOp):
+    r"""Sort list of dicts
+
+    Args:
+        col (int or tuple/list): The column number or list of columns as sorting criteria
+        reverse (bool): Reverse the sort (Default : False)
+
+    Returns:
+        list of lists: sorted input
+
+    Examples:
+        to come...
+    """
+    @classmethod
+    def op(cls,lists, col, reverse=False,*args,**kwargs):
+        if isinstance(col,(tuple,list)):
+            fn = lambda x:[ x[c] for c in col ]
+        else:
+            fn = lambda x: x[col]
+
+        return sorted(lists,key = fn, reverse=reverse)
+
+class sortdicts(TextOp):
+    r"""Sort list of dicts
+
+    Args:
+        key (int or tuple/list): The dict key or list of keys as sorting criteria
+        reverse (bool): Reverse the sort (Default : False)
+
+    Returns:
+        list of dicts: sorted input
+
+    Examples:
+        to come...
+    """
+    @classmethod
+    def op(cls,dicts, key, reverse=False,*args,**kwargs):
+        if isinstance(key,(tuple,list)):
+            fn = lambda x:[ x[k] for k in key ]
+        else:
+            fn = lambda x: x[key]
+
+        return sorted(dicts,key = fn, reverse = reverse)
+
 class dorender(TextOp):
     r"""Formats list of strings
 
@@ -1573,15 +1474,21 @@ class linetester(TextOp):
     @classmethod
     def op(cls, text, *args,**kwargs):
         key = kwargs.get('key')
+        attr = kwargs.get('attr')
         castfn = cls.castfn(*args,**kwargs)
 
-        if key is None:
-            getkey = lambda l:l
-        else:
+        if key is not None:
             if callable(key):
                 getkey = lambda l:key(StrExt(l))
             else:
                 getkey = lambda l:l[key]
+        elif attr is not None:
+            if callable(attr):
+                getkey = lambda l: attr(StrExt(l))
+            else:
+                getkey = lambda l: getattr(l,attr)
+        else:
+            getkey = lambda l: l
 
         for line in cls._tolist(text):
             try:
@@ -1590,7 +1497,7 @@ class linetester(TextOp):
                 if cls.testline(to_test, *args,**kwargs):
                     yield line
             except (ValueError, TypeError, IndexError, KeyError):
-                pass
+                raise
 
 class inrange(linetester):
     r"""Extract lines between a range of strings
@@ -2247,13 +2154,14 @@ class subitems(TextOp):
             yield [ sublist[n] for n in ntab ]
 
 class uniq(TextOp):
-    r"""Remove all line repetitions
+    r"""Remove all line repetitions at any place
 
     If a line is many times in the same text (even if there are some different lines between),
-    only the first will be taken. Works also with list of lists or dicts.
+    only the first will be taken. Works also with list of lists or dicts. Input order is preserved.
+    Do not use this operation on huge data set as an internal list is maintained : if possible, use :class:`textops.norepeat`.
 
     Returns:
-        generator: Unified text line by line.
+        generator: Uniqified text line by line.
 
     Examples:
         >>> s='f\na\nb\na\nc\nc\ne\na\nc\nf'
@@ -2281,6 +2189,34 @@ class uniq(TextOp):
                 s.append(line)
                 yield line
 
+class norepeat(TextOp):
+    r"""Remove line repetitions that follows
+
+    If a line is the same as the previous one, it is not yield. Works also with list of lists or dicts.
+    Input order is preserved and it can be used on huge files.
+
+    Returns:
+        generator: Unrepeated text line by line.
+
+    Examples:
+        >>> s='f\na\nb\na\nc\nc\ne\na\nc\nf'
+        >>> s >> norepeat()
+        ['f', 'a', 'b', 'a', 'c', 'e', 'a', 'c', 'f']
+        >>> l = [ [1,2], [3,4], [1,2], [1,2] ]
+        >>> l >> norepeat()
+        [[1, 2], [3, 4], [1, 2]]
+        >>> d = [ {'a':1}, {'b':2}, {'a':1}, {'a':1} ]
+        >>> d >> norepeat()
+        [{'a': 1}, {'b': 2}, {'a': 1}]
+    """
+    @classmethod
+    def op(cls, text, *args,**kwargs):
+        prev = None
+        for line in cls._tolist(text):
+            if line != prev:
+                prev=line
+                yield line
+
 class splitblock(TextOp):
     r"""split a text into blocks
 
